﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using Modbus.Utility;
using System.Xml.Linq;
using System.Runtime.CompilerServices;

namespace ZLAC8015_RS485_GUI
{
    public partial class Form1 : Form
    {
        private ZLAC8015_RS485_GUI.Modbus Modbus = new ZLAC8015_RS485_GUI.Modbus();

        private bool Serial_Connect_State = false;

        private bool Servo_OnOff_State = false;

        private byte Driver_ID;

        public ushort[] ReadData = new ushort[9];

        public int Voltage_High = 0;
        public int Voltage_Low = 0;
        public int Position_High = 0;
        public int Position_Low = 0;
        public int Velocty_High = 0;
        public int Velocty_Low = 0;
        public int Temperature_High = 0;
        public int Temperature_Low = 0;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                string[] Serial_Port_Name = SerialPort.GetPortNames();
                Comport_CB.Items.AddRange(Serial_Port_Name);
                Comport_CB.Text = Serial_Port_Name[0];
                Baudrate_CB.Text = "115200";
                Driver_ID = Convert.ToByte(Driver_ID_TB.Text);

                Connect_Btn.Text = "연결";
                Connect_Btn.BackColor = Color.Aquamarine;

                Connect_State_Label.ForeColor = Color.DarkOrange;
                Connect_State_Label.Text = "연결이 해제되었습니다.";

                Servo_On_Btn.BackColor = Color.Aquamarine;
                Servo_Off_Btn.BackColor = Color.DarkOrange;
            }
            catch (IndexOutOfRangeException) { }
        }

        private void Connect_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                String Comport = Comport_CB.SelectedItem.ToString();
                int Baudrate = Convert.ToInt32(Baudrate_CB.SelectedItem.ToString());
                if (!Serial_Connect_State)
                {
                    if (Modbus.Connect(Comport, Baudrate))
                    {
                        Serial_Connect_State = true;
                        Connect_Btn.Text = "해제";
                        Connect_Btn.BackColor = Color.Orange;
                        Connect_State_Label.ForeColor = Color.Blue;
                        Connect_State_Label.Text = "연결되었습니다.";
                        Read_Timer.Enabled = true;
                        //State_Read_Timer.Enabled = true;
                        //UI_Updata_Timer.Enabled = true;

                    }
                }
                else
                {
                    if (Modbus.DisConnect())
                    {
                        Serial_Connect_State = false;
                        Connect_Btn.Text = "연결";
                        Connect_Btn.BackColor = Color.LightGreen;
                        Connect_State_Label.ForeColor = Color.Orange;
                        Connect_State_Label.Text = "연결이 해제되었습니다.";
                        Read_Timer.Enabled = false;
                        //State_Read_Timer.Enabled = false;
                        //UI_Updata_Timer.Enabled = false;
                    }
                }
            }
            catch (Exception)
            {
                MessageBox.Show("컴포트 및 연결상태를 확인해주세요");
                DisConnect();
            }
        }
        private void DisConnect()
        {
            Modbus.DisConnect();
            Serial_Connect_State = false;
            Connect_Btn.Text = "연결";
            Connect_Btn.BackColor = Color.LightGreen;
            Connect_State_Label.ForeColor = Color.Orange;
            Connect_State_Label.Text = "연결이 해제되었습니다.";
            Read_Timer.Enabled = false;
            //State_Read_Timer.Enabled = false;
            //UI_Updata_Timer.Enabled = false;
        }

        private void Servo_On_Btn_Click(object sender, EventArgs e)
        {
            if (Serial_Connect_State)
            {
                Modbus.Servo_On(Driver_ID);
                Servo_OnOff_State = true;
            }
            else return;
        }

        private void Servo_Off_Btn_Click(object sender, EventArgs e)
        {
            if (Serial_Connect_State)
            {
                Modbus.Servo_Off(Driver_ID);
                Servo_OnOff_State = false;
            }
            else return;
        }

        private void Position_Start_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                int Position = Convert.ToInt32(Position_Position_TB.Text);
                ushort Speed = Convert.ToUInt16(Position_Speed_TB.Text);
                ushort ACC = Convert.ToUInt16(Position_ACC_CB.Text);
                ushort DEC = Convert.ToUInt16(Position_DEC_TB.Text);
                ushort CMD;
                switch (Position_Mode_CB.SelectedIndex)
                {
                    case 0:
                        CMD = 0x02; //절대 위치
                        break;
                    case 1:
                        CMD = 0x01; //상대 위치 
                        break;
                    default:
                        CMD = 0x00; //no action
                        break;
                }
                if (Serial_Connect_State)
                {
                    if (Servo_OnOff_State)
                    {
                        Modbus.Position_Motion(Driver_ID, Position, CMD, ACC, DEC, Speed);

                    }
                    else
                    {
                        MessageBox.Show("Servo ON 상태가 아닙니다.");
                    }
                }
            }
            catch (Exception ecx)
            {
                if (ecx.Message == "작업시간을 초과했습니다.")
                {
                    MessageBox.Show("컴포트 및 연결상태를 확인해주세요.");
                }
                else
                {
                    MessageBox.Show("위치, 속도, 가감속값을 확인해주세요.");
                }
            }
        }

        private void Position_Stop_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                if (Serial_Connect_State)
                {
                    Modbus.Position_Stop(Driver_ID);
                }
            }
            catch (TimeoutException)
            {
                MessageBox.Show("드라이버 ID 및 통신속도를 확인해주세요.");

            }
        }

        private void Velocity_Start_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                ushort ACC = Convert.ToUInt16(Velocity_ACC_TB.Text);
                ushort DEC = Convert.ToUInt16(Velocity_DEC_TB.Text);
                short Speed = Convert.ToInt16(Velocity_Speed_TB.Text);
                if (Serial_Connect_State)
                {
                    if (Serial_Connect_State)
                    {
                        Modbus.Velocity_Motion(Driver_ID, ACC, DEC, Speed);
                    }
                    else
                    {
                        MessageBox.Show("Servo ON 상태가 아닙니다.");
                    }
                }
            }
            catch (Exception exc)
            {
                if (exc.Message == "작업 시간을 초과했습니다.")
                {
                    MessageBox.Show("컴포트 및 연결상태를 확인해주세요.");
                }
                else
                {
                    MessageBox.Show("속도 및 가감속 값을 확인해주세요");
                }
            }
        }

        private void Velocity_Stop_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                if (Serial_Connect_State)
                {
                    Modbus.Velocity_Stop(Driver_ID);
                }
            }
            catch
            {
                MessageBox.Show("드라이버 ID 및 통신속도를 확인해주세요.");
            }
        }
        private void Torque_Start_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                ushort Slope = Convert.ToUInt16(Torque_Slope_TB.Text);
                short Torque = Convert.ToInt16(Torque_Torque_TB.Text);
                if (Serial_Connect_State)
                {
                    if (Servo_OnOff_State)
                    {
                        Modbus.Torque_Motion(Driver_ID, Slope, Torque);
                    }
                }
            }
            catch (Exception exc)
            {
                if (exc.Message == "작업 시간을 초과했습니다.")
                {
                    MessageBox.Show("컴포트 및 연결상태를 확인해주세요.");
                }
                else
                {
                    MessageBox.Show("토크, 가감속 값을 확인해주세요.");
                }
            }
        }
        private void Torque_Stop_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                if (Serial_Connect_State)
                {
                    Modbus.Torque_Stop(Driver_ID) ;
                }
            }
            catch
            {
                MessageBox.Show("드라이버 ID 및 통신속도를 확인해주세요.");
            }
        }
        private void Read_Timer_Tick(object sender, EventArgs e)
        {
            ReadData = Modbus.Read_IO_Parameter(Driver_ID);

            try
            {
                if (Serial_Connect_State)
                {
                    Voltage_High = BitConverter.ToInt16(BitConverter.GetBytes(ReadData[(char)Modbus.Read_Data_Mapping.Bus_Voltage]), 0) / 100;
                    Voltage_Low = BitConverter.ToInt16(BitConverter.GetBytes(ReadData[(char)Modbus.Read_Data_Mapping.Bus_Voltage]), 0) % 100;
                    State_Voltage_TB.Text = Voltage_High.ToString() + "." + Voltage_Low.ToString();
                    State_Position_TB.Text = BitConverter.ToInt32(BitConverter.GetBytes(ModbusUtility.GetUInt32(ReadData[(char)Modbus.Read_Data_Mapping.Position_High], ReadData[(char)Modbus.Read_Data_Mapping.Position_Low])), 0).ToString();
                    State_ALM_Code_TB.Text = "0x"+Convert.ToString(ReadData[(char)Modbus.Read_Data_Mapping.Error_Code], 16); //ReadData[(char)Modbus.Read_Data_Mapping.Error_Code].ToString();
                    Velocty_High = BitConverter.ToInt16(BitConverter.GetBytes(ReadData[(char)Modbus.Read_Data_Mapping.Motor_Speed]), 0) / 10;
                    State_Velocity_TB.Text = Velocty_High.ToString();
                    State_Torque_TB.Text = BitConverter.ToInt16(BitConverter.GetBytes(ReadData[(char)Modbus.Read_Data_Mapping.Motor_Torque]), 0).ToString();
                    Temperature_High = BitConverter.ToInt16(BitConverter.GetBytes(ReadData[(char)Modbus.Read_Data_Mapping.Motor_Temperature]), 0) / 10;
                    Temperature_Low = BitConverter.ToInt16(BitConverter.GetBytes(ReadData[(char)Modbus.Read_Data_Mapping.Motor_Temperature]), 0) % 10;
                    State_Temperature_TB.Text = Temperature_High.ToString() + "." + Temperature_Low.ToString();


                    if ((ReadData[(char)Modbus.Read_Data_Mapping.Error_Code]) == 0) { State_ALM_LED.BackColor = Color.DarkGray; }
                    else { State_ALM_LED.BackColor = Color.GreenYellow; }
                    if ((ReadData[(char)Modbus.Read_Data_Mapping.Motor_Status] & 0x01) != 0) { State_Running_LED.BackColor = Color.GreenYellow; }
                    else
                    { State_Running_LED.BackColor = Color.DarkGray; }
                    if (Servo_OnOff_State == true)
                    {
                        State_ENA_LED.BackColor = Color.GreenYellow;
                    }
                    else
                    {
                        State_ENA_LED.BackColor = Color.DarkGray;
                    }

                    if ((ReadData[(char)Modbus.Read_Data_Mapping.Hall_Status] & 0x01) != 0) { State_U_LED.BackColor = Color.GreenYellow; }
                    else
                    {
                        { State_U_LED.BackColor = Color.DarkGray; }
                    }
                    if ((ReadData[(char)Modbus.Read_Data_Mapping.Hall_Status] & 0x02) != 0) { State_V_LED.BackColor = Color.GreenYellow; }
                    else
                    {
                        { State_V_LED.BackColor = Color.DarkGray; }
                    }
                    if ((ReadData[(char)Modbus.Read_Data_Mapping.Hall_Status] & 0x04) != 0) { State_W_LED.BackColor = Color.GreenYellow; }
                    else
                    {
                        { State_W_LED.BackColor = Color.DarkGray; }
                    }
                }
            }
            catch (TimeoutException)
            {
                DisConnect();
                MessageBox.Show("컴포트 및 연결상태를 확인해주세요.");
            }
        }
    

        private void ALM_Clear_Btn_Click(object sender, EventArgs e)
        {
            try
            {
                if (Serial_Connect_State)
                {
                    Modbus.ALM_Clear(Driver_ID);
                }
            }
            catch 
            {
                MessageBox.Show("다시 확인해주세요.");
            }
        }

        private void Driver_ID_TB_TextChanged(object sender, EventArgs e)
        {
            try 
            {
                if (Serial_Connect_State)
                {

                    Read_Timer.Enabled = false;
                    Driver_ID = Convert.ToByte(Driver_ID_TB.Text);
                    if (Driver_ID > 0)
                    {
                        Read_Timer.Enabled = true;
                    }
                }
                else
                {
                    Driver_ID = Convert.ToByte(Driver_ID_TB.Text);
                }
            }
            catch(Exception)
            {

            }
        }

        private void Position_Reset_Click(object sender, EventArgs e)
        {
            try
            {
                if (Serial_Connect_State)
                {
                    Modbus.Position_Reset(Driver_ID);
                }
            }
            catch
            {
                MessageBox.Show("다시 확인해주세요.");
            }
        }
    }

    }

